==> dart_icalendar-1.0.2.tar.gz: lib/models/free_busy_model.dart <==
import 'dart:convert';

import 'package:collection/collection.dart';

class FreeBusy {
  final DateTime start;
  final DateTime end;
  final List<TimeRange> busy;

  FreeBusy({
    required this.start,
    required this.end,
    required this.busy,
  });

  factory FreeBusy.fromICalString(String icalString) {
    final lines = icalString.split('\n');
    DateTime? start;
    DateTime? end;
    List<TimeRange>? busy;
    for (final line in lines) {
      if (line.startsWith('DTSTART:')) {
        start = DateTime.parse(line.substring(8));
      } else if (line.startsWith('DTEND:')) {
        end = DateTime.parse(line.substring(6));
      } else if (line.startsWith('FREEBUSY:')) {
        final parts = line.substring(10).split('/');
        busy = parts.map((part) => TimeRange.fromICalString(part)).toList();
      }
    }
    if (start == null || end == null || busy == null) {
      throw FormatException('Invalid iCalendar string: $icalString');
    }
    return FreeBusy(start: start, end: end, busy: busy);
  }

  String toICalString() {
    final buffer = StringBuffer();
    buffer.writeln('BEGIN:VFREEBUSY');
    buffer.writeln('DTSTART:${start.toUtc().toIso8601String()}Z');
    buffer.writeln('DTEND:${end.toUtc().toIso8601String()}Z');
    buffer.writeln(
        'FREEBUSY:${busy.map((range) => range.toICalString()).join('/')}');
    buffer.writeln('END:VFREEBUSY');
    return buffer.toString();
  }

  FreeBusy copyWith({
    DateTime? start,
    DateTime? end,
    List<TimeRange>? busy,
  }) {
    return FreeBusy(
      start: start ?? this.start,
      end: end ?? this.end,
      busy: busy ?? this.busy,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'start': start.millisecondsSinceEpoch,
      'end': end.millisecondsSinceEpoch,
      'busy': busy.map((x) => x.toMap()).toList(),
    };
  }

  factory FreeBusy.fromMap(Map<String, dynamic> map) {
    return FreeBusy(
      start: DateTime.fromMillisecondsSinceEpoch(map['start']),
      end: DateTime.fromMillisecondsSinceEpoch(map['end']),
      busy: List<TimeRange>.from(map['busy']?.map((x) => TimeRange.fromMap(x))),
    );
  }

  String toJson() => json.encode(toMap());

  factory FreeBusy.fromJson(String source) =>
      FreeBusy.fromMap(json.decode(source));

  @override
  String toString() => 'FreeBusy(start: $start, end: $end, busy: $busy)';

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    final listEquals = const DeepCollectionEquality().equals;

    return other is FreeBusy &&
        other.start == start &&
        other.end == end &&
        listEquals(other.busy, busy);
  }

  @override
  int get hashCode => start.hashCode ^ end.hashCode ^ busy.hashCode;
}

class TimeRange {
  final DateTime start;
  final DateTime end;

  TimeRange({
    required this.start,
    required this.end,
  });

  factory TimeRange.fromICalString(String icalString) {
    final parts = icalString.split('/');
    final start = DateTime.parse(parts[0]);
    final end = DateTime.parse(parts[1]);
    return TimeRange(start: start, end: end);
  }

  String toICalString() {
    return '${start.toUtc().toIso8601String()}Z/${end.toUtc().toIso8601String()}Z';
  }

  TimeRange copyWith({
    DateTime? start,
    DateTime? end,
  }) {
    return TimeRange(
      start: start ?? this.start,
      end: end ?? this.end,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'start': start.millisecondsSinceEpoch,
      'end': end.millisecondsSinceEpoch,
    };
  }

  factory TimeRange.fromMap(Map<String, dynamic> map) {
    return TimeRange(
      start: DateTime.fromMillisecondsSinceEpoch(map['start']),
      end: DateTime.fromMillisecondsSinceEpoch(map['end']),
    );
  }

  String toJson() => json.encode(toMap());

  factory TimeRange.fromJson(String source) =>
      TimeRange.fromMap(json.decode(source));

  @override
  String toString() => 'TimeRange(start: $start, end: $end)';

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is TimeRange && other.start == start && other.end == end;
  }

  @override
  int get hashCode => start.hashCode ^ end.hashCode;
}

==> dart_icalendar-1.0.14.tar.gz: lib/models/free_busy_model.dart <==
import 'dart:convert';

import 'package:collection/collection.dart';

class FreeBusy {
  DateTime start;
  DateTime end;
  List<TimeRange> busy;

  FreeBusy({
    required this.start,
    required this.end,
    required this.busy,
  });

  factory FreeBusy.fromICalString(String icalString) {
    final lines = icalString.split('\n');
    DateTime? start;
    DateTime? end;
    List<TimeRange>? busy;
    for (final line in lines) {
      if (line.startsWith('DTSTART:')) {
        start = DateTime.parse(line.substring(8));
      } else if (line.startsWith('DTEND:')) {
        end = DateTime.parse(line.substring(6));
      } else if (line.startsWith('FREEBUSY:')) {
        final parts = line.substring(10).split('/');
        busy = parts.map((part) => TimeRange.fromICalString(part)).toList();
      }
    }
    if (start == null || end == null || busy == null) {
      throw FormatException('Invalid iCalendar string: $icalString');
    }
    return FreeBusy(start: start, end: end, busy: busy);
  }

  String toICalString() {
    final buffer = StringBuffer();
    buffer.writeln('BEGIN:VFREEBUSY');
    buffer.writeln('DTSTART:${start.toUtc().toIso8601String()}Z');
    buffer.writeln('DTEND:${end.toUtc().toIso8601String()}Z');
    buffer.writeln(
        'FREEBUSY:${busy.map((range) => range.toICalString()).join('/')}');
    buffer.writeln('END:VFREEBUSY');
    return buffer.toString();
  }

  FreeBusy copyWith({
    DateTime? start,
    DateTime? end,
    List<TimeRange>? busy,
  }) {
    return FreeBusy(
      start: start ?? this.start,
      end: end ?? this.end,
      busy: busy ?? this.busy,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'start': start.millisecondsSinceEpoch,
      'end': end.millisecondsSinceEpoch,
      'busy': busy.map((x) => x.toMap()).toList(),
    };
  }

  factory FreeBusy.fromMap(Map<String, dynamic> map) {
    return FreeBusy(
      start: DateTime.fromMillisecondsSinceEpoch(
          int.parse(map['start'].toString())),
      end:
          DateTime.fromMillisecondsSinceEpoch(int.parse(map['end'].toString())),
      busy: List<TimeRange>.from(map['busy']?.map((x) => TimeRange.fromMap(x))),
    );
  }

  String toJson() => json.encode(toMap());

  factory FreeBusy.fromJson(String source) =>
      FreeBusy.fromMap(json.decode(source));

  @override
  String toString() => 'FreeBusy(start: $start, end: $end, busy: $busy)';

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    final listEquals = const DeepCollectionEquality().equals;

    return other is FreeBusy &&
        other.start == start &&
        other.end == end &&
        listEquals(other.busy, busy);
  }

  @override
  int get hashCode => start.hashCode ^ end.hashCode ^ busy.hashCode;
}

class TimeRange {
  final DateTime start;
  final DateTime end;

  TimeRange({
    required this.start,
    required this.end,
  });

  factory TimeRange.fromICalString(String icalString) {
    final parts = icalString.split('/');
    final start = DateTime.parse(parts[0]);
    final end = DateTime.parse(parts[1]);
    return TimeRange(start: start, end: end);
  }

  String toICalString() {
    return '${start.toUtc().toIso8601String()}Z/${end.toUtc().toIso8601String()}Z';
  }

  TimeRange copyWith({
    DateTime? start,
    DateTime? end,
  }) {
    return TimeRange(
      start: start ?? this.start,
      end: end ?? this.end,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'start': start.millisecondsSinceEpoch,
      'end': end.millisecondsSinceEpoch,
    };
  }

  factory TimeRange.fromMap(Map<String, dynamic> map) {
    return TimeRange(
      start: DateTime.fromMillisecondsSinceEpoch(
          int.parse(map['start'].toString())),
      end:
          DateTime.fromMillisecondsSinceEpoch(int.parse(map['end'].toString())),
    );
  }

  String toJson() => json.encode(toMap());

  factory TimeRange.fromJson(String source) =>
      TimeRange.fromMap(json.decode(source));

  @override
  String toString() => 'TimeRange(start: $start, end: $end)';

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is TimeRange && other.start == start && other.end == end;
  }

  @override
  int get hashCode => start.hashCode ^ end.hashCode;
}
